@@ -1,15 +1,18 @@
 """Module for reprsenting scheduler jobs."""
+import base64
 from datetime import timedelta
+import gzip
+import json
 import re
-from typing import Any, Dict, Optional, Union
+from typing import Any, Dict, Generator, Optional, Union
 
 
 multiple_map = {
-    "K": 1024 ** 0,
-    "M": 1024 ** 1,
-    "G": 1024 ** 2,
-    "T": 1024 ** 3,
-    "E": 1024 ** 4,
+    "K": 1024**0,
+    "M": 1024**1,
+    "G": 1024**2,
+    "T": 1024**3,
+    "E": 1024**4,
 }
 
 state_colors = {
@@ -59,7 +62,13 @@ class Job:
         self.cpu: Optional[Union[str, float]] = "---"
         self.mem: Union[str, float] = "---"
         self.state: Optional[str] = None
-        self.other_entries: Dict[str, str] = {}
+        self.mem_eff: Optional[float] = None
+        self.gpu: Optional[float] = None
+        self.gpu_mem: Optional[float] = None
+        self.other_entries: Dict[str, Any] = {}
+        # safe to cache now
+        self.other_entries["JobID"] = self.name()
+        self.comment_data: Dict = {}
 
     def __eq__(self, other: Any) -> bool:
         """Test for equality.
@@ -93,11 +102,13 @@ class Job:
             self.state = entry["State"].split()[0]
 
         if self.state == "PENDING":
+            self._cache_entries()
             return
 
         # main job id
         if self.jobid == entry["JobID"]:
             self._update_main_job(entry)
+            self._cache_entries()
 
         elif self.state != "RUNNING":
             for k, value in entry.items():
@@ -109,9 +120,11 @@ class Job:
         """Update properties for the main job.
 
         Args:
-            entry: the entry where the jobid matches exactly
+            entry: the entry where the jobid matches exactly, e.g. not batch or ex
         """
-        self.other_entries = entry
+        for k, value in entry.items():
+            if k not in self.other_entries or not self.other_entries[k]:
+                self.other_entries[k] = value
         self.time = entry["Elapsed"] if "Elapsed" in entry else None
         requested = (
             _parse_slurm_timedelta(entry["Timelimit"]) if "Timelimit" in entry else 1
@@ -139,6 +152,79 @@ class Job:
                 entry["REQMEM"], int(entry["NNodes"]), int(entry["AllocCPUS"])
             )
 
+        if "AdminComment" in entry and len(entry["AdminComment"]) > 10:
+            self._parse_admin_comment(entry["AdminComment"])
+
+    def _parse_admin_comment(self, comment: str) -> None:
+        """Use admin command to override efficiency values.
+
+        Decodes and parses admincommand from jobstats.
+
+        Args:
+            comment: The AdminComment field.
+
+        Raises:
+            ValueError: if the comment doesn't start with JS1.
+            ValueError: if the comment can't be decoded.
+        """
+        comment_type = comment[:3]
+        if comment_type not in ("JS1",):
+            raise ValueError(f"Unknown comment type '{comment_type}'")
+        data = {}
+        try:
+            data = json.loads(gzip.decompress(base64.b64decode(comment[4:])))
+        except BaseException as exception:
+            raise ValueError(f"Cannot decode comment '{comment}'") from exception
+
+        def average(value: str, data: Optional[dict] = None) -> float:
+            if data is None:
+                data = self.comment_data
+            return round(
+                sum(v[value] for v in data.values()) / len(data),
+                1,
+            )
+
+        for node, value in data["nodes"].items():
+            self.comment_data[node] = {
+                "CPUEff": value["total_time"]
+                / value["cpus"]
+                / data["total_time"]
+                * 100,
+                "MemEff": value["used_memory"] / value["total_memory"] * 100,
+            }
+            if data["gpus"]:
+                self.comment_data[node]["gpus"] = {
+                    gpu: {
+                        "GPUEff": value["gpu_utilization"][gpu],
+                        "GPUMem": round(
+                            value["gpu_used_memory"][gpu]
+                            / value["gpu_total_memory"][gpu]
+                            * 100,
+                            1,
+                        ),
+                    }
+                    for gpu in value["gpu_utilization"]
+                }
+                self.comment_data[node]["GPUEff"] = average(
+                    "GPUEff", self.comment_data[node]["gpus"]
+                )
+                self.comment_data[node]["GPUMem"] = average(
+                    "GPUMem", self.comment_data[node]["gpus"]
+                )
+
+        self.cpu = average("CPUEff")
+        self.mem_eff = average("MemEff")
+        if data["gpus"]:
+            self.gpu = average("GPUEff")
+            self.gpu_mem = average("GPUMem")
+
+    def _cache_entries(self) -> None:
+        self.other_entries["State"] = self.state
+        self.other_entries["TimeEff"] = self.time_eff
+        self.other_entries["CPUEff"] = self.cpu if self.cpu else "---"
+        self.other_entries["GPUEff"] = self.gpu if self.gpu else "---"
+        self.other_entries["GPUMem"] = self.gpu_mem if self.gpu_mem else "---"
+
     def name(self) -> str:
         """The name of the job.
 
@@ -158,20 +244,51 @@ class Job:
         Returns:
             The value of that attribute or "---" if not found
         """
-        if key == "JobID":
-            return self.name()
-        if key == "State":
-            return self.state
         if key == "MemEff":
+            if self.mem_eff:  # set by admin comment
+                return self.mem_eff
             if self.totalmem:
                 return round(self.stepmem / self.totalmem * 100, 1)
             return "---"
-        if key == "TimeEff":
-            return self.time_eff
-        if key == "CPUEff":
-            return self.cpu if self.cpu else "---"
-        else:
-            return self.other_entries.get(key, "---")
+        return self.other_entries.get(key, "---")
+
+    def get_node_entries(
+        self, key: str, gpu: bool = False
+    ) -> Generator[Any, None, None]:
+        """Get an attribute by name for each node in job.
+
+        Args:
+            key: the attribute to query
+            gpu: True if gpu values should also be split
+
+        Yields:
+            A generator with values for provided attribute.  If the key does
+            not change in nodes/gpus it will yield an empty string
+        """
+        yield self.get_entry(key)
+        if len(self.comment_data) > 1 or (gpu and self.gpu):
+            for node, data in self.comment_data.items():
+                # get node-level data
+                if key == "JobID":
+                    yield f"  {node}"
+                else:
+                    to_yield = data.get(key, "")
+                    to_yield = (
+                        to_yield if isinstance(to_yield, str) else round(to_yield, 1)
+                    )
+                    yield to_yield
+                if gpu and self.gpu:  # has gpus to report
+                    for gpu_name, gpu_data in data["gpus"].items():
+                        if key == "JobID":
+                            yield f"    {gpu_name}"
+                        else:
+                            to_yield = gpu_data.get(key, "")
+                            to_yield = (
+                                to_yield
+                                if isinstance(to_yield, str)
+                                else round(to_yield, 1)
+                            )
+                            yield to_yield
 
 
 def _parse_slurm_timedelta(delta: str) -> int:
